#include "sha1.h"

// change to your secret
uint8_t hmacKey1[]={ 0x48, 0x65, 0x6c, 0x6c, 0x6f, 0x21, 0xde, 0xad, 0xbe, 0xef };

void mode_totp(uint8_t action) {
  DateTime now;
  uint8_t  i, dx, dw, y, h;
  int      x, sum;
  long     epoch;

  if(action != ACTION_NONE) {

    // If we just arrived here (whether through mode change
    // or wake from sleep), initialize the matrix driver:
    if(action >= ACTION_HOLD_LEFT) {
      uint8_t depth = 7, plex = LED_PLEX_4;
      // Reduce depth/plex if battery voltage is low
      if(watch.getmV() < BATT_LOW_MV) {
        depth = 2;
        plex  = LED_PLEX_1;
      }
      // Reconfigure display if needed
      if((watch.getDepth() != depth) || (watch.getPlex() != plex))
        fps = watch.setDisplayMode(depth, plex, true);
      // And begin in time-display mode:
    }

    // Load time/date digits depending on current submode:
    now = RTC.now();
    len = 0;
    epoch = now.unixtime() - 3600;
    uint8_t byteArray[8];   
    long time = epoch / 30;
    
    byteArray[0] = 0x00;
    byteArray[1] = 0x00;
    byteArray[2] = 0x00;
    byteArray[3] = 0x00;
    byteArray[4] = (int)((time >> 24) & 0xFF) ;
    byteArray[5] = (int)((time >> 16) & 0xFF) ;
    byteArray[6] = (int)((time >> 8) & 0XFF);
    byteArray[7] = (int)((time & 0XFF));
    
    uint8_t* hash;
    uint32_t a; 
    Sha1.initHmac(hmacKey1,sizeof(hmacKey1));
    Sha1.writebytes(byteArray, 8);
    hash = Sha1.resultHmac();
  
    int  offset = hash[20 - 1] & 0xF; 
    long truncatedHash = 0;
    int j;
    for (j = 0; j < 4; ++j) {
     truncatedHash <<= 8;
     truncatedHash  |= hash[offset + j];
    }
    
    truncatedHash &= 0x7FFFFFFF;
    truncatedHash %= 1000000;
    
    for (len = 5; len >= 0; len--) {
      str[len] = truncatedHash % 10;
      truncatedHash /= 10;
    }
    len = 6;

    sum = 0;
    for(i=0; i<len; i++) sum += pgm_read_byte(&digitWidth[str[i]]);
    // When in power-reduced mode, blitting happens quicker because
    // of fewer screen bits; scale the timeout to compensate.
    if(watch.getPlex() != LED_PLEX_4) sum *= 3;

    // Reset sleep timeout on ANY button action
    watch.setTimeout((sum + 8) * 4); // Sleep after time/date scrolls off left edge

    curX = 8; // Initialize position off right edge
    f    = 0;
  }

  watch.fillScreen(0);
  x = curX;
  h = 8 - (f * 2); // Height of upper section of character to blit (2-8)
  y = f * 8;       // Vertical position of character in marqueeDigits[].
  // There are actually 4 bitmaps of each digit, each offset by 1/4 pixel
  // to provide subpixel antialiasing when scrolling -- extra smoothness!
  for(i=0; i<len; i++) {
    dx = pgm_read_byte(&digitX[str[i]]);
    dw = pgm_read_byte(&digitWidth[str[i]]);
    // The blit() function doesn't do real compositing -- it overwrites.
    // In order to pack the italicized digits closer together, they're
    // blitted as an upper and lower half (offset by 1 pixel) that don't
    // overlap the prior character.
    blit(          marqueeDigits, 63, 32, dx  , y  , x  , 0, dw, h  , 255 );
    if(h < 8) blit(marqueeDigits, 63, 32, dx-1, y+h, x-1, h, dw, 8-h, 255 );
    x += pgm_read_byte(&digitWidth[str[i]]);
  }

  // Every 4th frame, move left 1 pixel.  X is not reset or wrapped here...
  // timeout will kick in, and X will be set on the next wake operation.
  if(++f >= 4) {
    curX--;
    f = 0;
  }
}

